/***********************************
 *send83 v0.w by David Lennartsson *
 *f97dale@dd.chalmers.se           *
 *98-07-22                         *
 *included in tilin02.tgz and      *
 *released under GNU Public License*
 ***********************************/


#include <stdio.h>
#include <stdlib.h>

#define _TIMEOUT	500000

FILE *dev;

sendEOT()
{
	fputc(0x03,dev);
	fputc(0x92,dev);
	fputc(0x0b,dev);
	fputc(0x00,dev);
}

sendACK()
{
	fputc(0x03,dev);
	fputc(0x56,dev);
	fputc(0x0b,dev);
	fputc(0x00,dev);
}

int waitACK()
{
	unsigned char chr;
	long int timeout;
	timeout=_TIMEOUT;
	fread(&chr,1,1,dev);
	while(chr!=0x83)
	{
		fread(&chr,1,1,dev);
		timeout--;
		if(timeout==0)
			return(5);
	}	
	while(chr!=0x56)
	{
		fread(&chr,1,1,dev);
		timeout--;
		if(timeout==0)
			return(5);
		if(chr==0x5A)
		{
			fread(&chr,1,1,dev);
			fread(&chr,1,1,dev);
			return(1); //Chksumerrorrec
		}
	}
	fread(&chr,1,1,dev);
	fread(&chr,1,1,dev);
	printf("Ack received\n");
	return(0);
}

waitCONT()
{
	unsigned char chr;
	long int timeout;
	timeout=_TIMEOUT;
	fread(&chr,1,1,dev);
	while(chr!=0x83)
	{
		fread(&chr,1,1,dev);
		timeout--;
		if(timeout==0)
			return(5);
	}
	while(chr!=0x09)
	{
		fread(&chr,1,1,dev);
		timeout--;
		if(timeout==0)
			return(5);
		if(chr==0x36)
		{
			fread(&chr,1,1,dev);
			fread(&chr,1,1,dev);
			fread(&chr,1,1,dev);
			fread(&chr,1,1,dev);
			fread(&chr,1,1,dev);
			return(2); //exit or skip recieved
		}
	}
	fread(&chr,1,1,dev);
	fread(&chr,1,1,dev);
	printf("Continue command received\n");
	return(0);
}

int calcchksum(unsigned char *data,int length)
{
	unsigned int count,chk;
	for(count=chk=0;count<length;count++)
	{
		chk+=data[count];
	}
	return(chk%65536);
}

void send83head(char *data,int length)
{
	int count;

	/* Sending MID=Comp83, COM=VarHead and Datalength */
	fputc(0x03,dev);
	fputc(0x06,dev);
	fputc(length&255,dev);
	fputc((length>>8)&255,dev);

	/* Sending header data */
	for(count=0;count<length;count++)
		fwrite(&data[count],1,1,dev);

	/* Calculate Checksum and send it */
	count=calcchksum(data,length);
	fputc(count&255,dev);
	fputc((count>>8)&255,dev);
}
void send83data(char *data,int length)
{
	unsigned char chr;
	int count;
	fputc(0x03,dev);
	fputc(0x15,dev);
	fputc(length&255,dev);
	fputc((length>>8)&255,dev);

	for(count=0;count<length;count++)
		fwrite(&data[count],1,1,dev);
	
	count=calcchksum(data,length);
	fputc(count&255,dev);
	fputc((count>>8)&255,dev);
}

int send83p(char *filename)
{
	FILE *infile;
	char string[50];
	unsigned char block[32000];
	unsigned char chr;
	int tmp,count,errorcode;
	tmp=count=0;
	

	if( (infile=fopen(filename,"r"))==NULL)
	{
		printf("Couldn't open file %s for input\n",filename);
		return(7);
	}
	printf("File opened\n");
	fread(string,8,1,infile);
	string[8]=0;
	printf("File Header: %s \n",string);
	printf("File Code: %x %x %x\n",fgetc(infile),fgetc(infile),fgetc(infile));
	fread(string,42,1,infile);
	string[42]=0;
	printf("File Comment: %s\n",string);
	fread(&tmp,2,1,infile);
	printf("File Length: %d \n",tmp);

	fread(&tmp,2,1,infile);

sending:

	fread(string,1,11,infile);
	send83head(string,11);
	printf("Program Header sent\n");

	errorcode=waitACK();
	if(errorcode)
		if(errorcode==5)
		{
			printf("Timed out waiting for Ack\n");
			return(5);
		}
		else
		{
			printf("Checksum error exiting\n");
			return(5);
		}

	errorcode=waitCONT();
	if(errorcode)
		if(errorcode==5)
		{
			printf("Timed out waiting for Continue command\n");
			return(5);
		}
		else
		{
			printf("Skip/Exit received, Exiting\n");
			return(5);
		}

	usleep(20);

	sendACK();
	printf("Sent Ack\n");

	fread(&tmp,2,1,infile);
	fread(block,1,tmp,infile);
	block[tmp]=0;


resenddata:
	send83data(block,tmp);

	printf("Datablock sent\n");
	errorcode=waitACK();
	if(errorcode)
		if(errorcode==5)
		{
			printf("Timed out waiting for Ack\n");
			return(5);
		}
		else
		{
			printf("Checksum error, resending data\n");
			goto resenddata;
		}


	fread(&tmp,2,1,infile);
	if(tmp!=11)
	{
		sendEOT();
		printf("EOT Sent\n");
		errorcode=waitACK();
		if(errorcode)
			if(errorcode==5)
			{
				printf("Timed out waiting for Ack\n");
				return(5);
			}
			else
			{
				printf("Checksum error, Exiting\n");
				return(5);
			}

		fgetc(infile);
		if(feof(infile))
		{
			printf("Linking completed!\n");
		}
		else
		{
			printf("Strange end of file!\n");
			return(3);
		}
		fclose(infile);
		return(0);
	}
	else
	{
		printf("Look! I found another file!\n");
		goto sending;
	}
}



int main(int argc,char *argv[])
{
	if((dev=fopen("/dev/ti","r+b"))!=NULL)
	{
		if(argc==2)
		{
			return(send83p(argv[1]));
		}
		else
		{
			printf("send83: Wrong syntax!\nUsage: %s filename\n",argv[0]);
			return(5);
		}
	}
	else
	{
		printf("send83: Error Opening /dev/ti !!\n");
		return(5);
	}
}
